package co.jp.mamol.myapp.controller;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Optional;

import co.jp.mamol.myapp.common.ApiResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import co.jp.mamol.myapp.model.CategoryModel;
import co.jp.mamol.myapp.model.SizaiModel;
import co.jp.mamol.myapp.model.UserModel;
import co.jp.mamol.myapp.form.BuyRequestForm;
import co.jp.mamol.myapp.service.BuyRequestService;

import jakarta.servlet.http.HttpSession;

@RestController
@RequestMapping("/api")
public class BuyRequestController {

	private final BuyRequestService buyRequestService;

	@Autowired
	public BuyRequestController(BuyRequestService buyRequestService) {
		this.buyRequestService = buyRequestService;
	}

	private UserModel getLoginUser(HttpSession session) {

		return (UserModel) session.getAttribute("loginInfo");
	}

	// M0002,BL1003,DA1003
	@GetMapping("/requestList/init")
	public ResponseEntity<ApiResponse<BuyRequestForm>> init(HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ユーザー情報は取らなかったですので、もう一度ログインしてください。"));
		}

		LocalDate endDate = LocalDate.now();
		LocalDate startDate = endDate.minusMonths(1);
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		String startDateStr = startDate.format(formatter);
		String endDateStr = endDate.format(formatter);

		List<SizaiModel> sizaiList = buyRequestService.getUserRequsetList(startDateStr, endDateStr, user.getId());
		BuyRequestForm buyRequestForm = new BuyRequestForm();
		if (sizaiList == null || sizaiList.isEmpty()) {
			ResponseEntity.badRequest().body(ApiResponse.fail("指定期間の購入依頼が登録されていません"));
		} else {
			buyRequestForm.setSizaiList(sizaiList);
		}
		buyRequestForm.setStartDate(startDateStr);
		buyRequestForm.setEndDate(endDateStr);
		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0002,BL1003,DA1003
	@PostMapping("/requestList/search")
	public ResponseEntity<ApiResponse<BuyRequestForm>> search(@RequestBody BuyRequestForm buyRequestForm, HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ユーザー情報は取らなかったですので、もう一度ログインしてください。"));
		}

		List<SizaiModel> sizaiList = buyRequestService.getUserRequsetList(buyRequestForm.getStartDate(), buyRequestForm.getEndDate(), user.getId());
		if (sizaiList == null || sizaiList.isEmpty()) {
			ResponseEntity.badRequest().body(ApiResponse.fail("指定期間の購入依頼が登録されていません"));
		} else {
			buyRequestForm.setSizaiList(sizaiList);
		}
		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0002,BL1006,DA1006
	@PostMapping("/requestList/delete")
	public ResponseEntity<ApiResponse<BuyRequestForm>> delete(@RequestBody BuyRequestForm buyRequestForm, HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
		}

		buyRequestService.deleteById(buyRequestForm.getDeleteById());

		String startDateStr = buyRequestForm.getStartDate();
		String endDateStr = buyRequestForm.getEndDate();
		if (startDateStr == null || endDateStr == null) {
			DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
			startDateStr = LocalDate.now().minusMonths(1).format(formatter);
			endDateStr = LocalDate.now().format(formatter);
		}

		List<SizaiModel> sizaiList = buyRequestService.getUserRequsetList(startDateStr, endDateStr, user.getId());
		buyRequestForm.setSizaiList(sizaiList);
		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0001,BL1002,DA1002
	@GetMapping("/requestRegist/init")
	public ResponseEntity<ApiResponse<BuyRequestForm>> registInit(HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
		}

		List<CategoryModel> categoryList = buyRequestService.getCategory();
		BuyRequestForm buyRequestForm = new BuyRequestForm();
		if (categoryList == null || categoryList.isEmpty()) {
			ResponseEntity.badRequest().body(ApiResponse.fail("申し訳ございません。今操作出来ません。"));
		} else {
			buyRequestForm.setCategoryModelList(categoryList);
		}
		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0001,BL1002,DA1002
	@PostMapping("/requestRegist/regist")
	public ResponseEntity<ApiResponse<BuyRequestForm>> regist(@RequestBody BuyRequestForm buyRequestForm, HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			ResponseEntity.status(403).body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
			return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
		}

		SizaiModel addSizai = buyRequestForm.getSizai();
		if (addSizai.getName() == null || addSizai.getName().isEmpty()) {

			return ResponseEntity.badRequest().body(ApiResponse.fail("資材名を入力してください。"));
		}
		if (addSizai.getNum() == 0) {

			return ResponseEntity.badRequest().body(ApiResponse.fail("数量を入力してください。"));
		}else if (addSizai.getNum() < 0) {

			return ResponseEntity.badRequest().body(ApiResponse.fail("数量はマイナスになりません。"));
		}

		addSizai.setRequest_user_id(user.getId());
		addSizai.setRequest_dept_id(user.getDepid());
		buyRequestService.requestRegist(addSizai);

		buyRequestForm.setCategoryModelList(buyRequestService.getCategory());

		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0003,BL1005,DA1005
	@GetMapping("/requestDetail/init")
	public ResponseEntity<ApiResponse<BuyRequestForm>> detailInit(@RequestParam("targetId") int targetId, HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
		}

		BuyRequestForm buyRequestForm = new BuyRequestForm();
		SizaiModel sizaiDetail = buyRequestService.getRequest(targetId);
		session.setAttribute("sizaiId", Optional.of(sizaiDetail.getId())); // 设置 session 数据
		buyRequestForm.setSizai(sizaiDetail);
		buyRequestForm.setCategoryModelList(buyRequestService.getCategory());

		if ("1".equals(sizaiDetail.getStatus()) || "3".equals(sizaiDetail.getStatus())) {
			buyRequestForm.setModifyFlag(true);
		} else {
			buyRequestForm.setModifyFlag(false);
			ResponseEntity.badRequest().body(ApiResponse.fail("承認済の依頼を変更できません。"));
		}
		return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
	}

	// M0003,BL1005,DA1005
	@PostMapping("/requestDetail/modify")
	public ResponseEntity<ApiResponse<BuyRequestForm>> modify(@RequestBody BuyRequestForm buyRequestForm, HttpSession session) {
		UserModel user = getLoginUser(session);
		if (user == null || !"1".equals(user.getRole_class())) {
			return ResponseEntity.status(403).body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
		}

		int sizaiId = buyRequestForm.getSizai().getId();
		SizaiModel sizai = buyRequestForm.getSizai();
		sizai.setId(sizaiId);

		boolean modified = buyRequestService.modifyRequest(sizai);
		if (modified) {
			buyRequestForm.setSizai(buyRequestService.getRequest(sizaiId));
			buyRequestForm.setCategoryModelList(buyRequestService.getCategory());
			buyRequestForm.setModifyFlag(true);
			return ResponseEntity.ok(ApiResponse.success(buyRequestForm));
		} else {
			return ResponseEntity.ok(ApiResponse.fail("変更できませんでした。"));
		}
	}
}