package co.jp.mamol.myapp.controller;

import co.jp.mamol.myapp.common.ApiResponse;
import co.jp.mamol.myapp.form.BuyApprovalForm;
import co.jp.mamol.myapp.form.OrderDeliverForm;
import co.jp.mamol.myapp.model.DepartmentModel;
import co.jp.mamol.myapp.model.SizaiModel;
import co.jp.mamol.myapp.model.UserModel;
import co.jp.mamol.myapp.service.OrderDeliverService;
import jakarta.servlet.http.HttpSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api")
public class OrderDeliverController {

    private final OrderDeliverService orderDeliverService;

    @Autowired
    public OrderDeliverController(OrderDeliverService orderDeliverService) {
        this.orderDeliverService = orderDeliverService;
    }

    private UserModel getLoginUser(HttpSession session) {

        return (UserModel) session.getAttribute("loginInfo");
    }

    @GetMapping("/order/init")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> orderInit(HttpSession session){
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        List<DepartmentModel> deptList = orderDeliverService.deptList();
        OrderDeliverForm orderDeliverForm = new OrderDeliverForm();
        orderDeliverForm.setDeptList(deptList);

        final String DEFAULT_DEPT_ID = "0001";
        List<SizaiModel> sizaiList = orderDeliverService.approvaleList(DEFAULT_DEPT_ID);
        orderDeliverForm.setSizaiList(sizaiList);

        if (orderDeliverForm.getSizaiList() == null || orderDeliverForm.getSizaiList().isEmpty()) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("指定部署の購入資材はありません。"));
        }else {

            session.setAttribute("deptId", orderDeliverForm.getDeptId());
            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @PostMapping("/order/search")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> orderSearch(@RequestBody OrderDeliverForm orderDeliverForm, HttpSession session){
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        List<DepartmentModel> deptList = orderDeliverService.deptList();
        orderDeliverForm.setDeptList(deptList);

        List<SizaiModel> sizaiList = orderDeliverService.approvaleList(orderDeliverForm.getDeptId());
        orderDeliverForm.setSizaiList(sizaiList);

        if (orderDeliverForm.getSizaiList() == null || orderDeliverForm.getSizaiList().isEmpty()) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("指定部署の購入資材はありません。"));
        }else {

            session.setAttribute("deptId", orderDeliverForm.getDeptId());
            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @PostMapping("/order/act")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> orderAct(@RequestBody OrderDeliverForm orderDeliverForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        String deptIdStr = orderDeliverForm.getDeptId();
        if (deptIdStr == null || deptIdStr.isBlank()) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("部署IDが指定されていません。"));
        }

        boolean flag = orderDeliverService.orderAct(orderDeliverForm.getId());

        if (!flag) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("発注失敗しました。"));
        }else {
            orderDeliverForm.setDeptId((String) session.getAttribute("deptId"));
            List<DepartmentModel> deptList = orderDeliverService.deptList();
            List<SizaiModel> sizaiList = orderDeliverService.approvaleList(orderDeliverForm.getDeptId());

            orderDeliverForm.setDeptList(deptList);
            orderDeliverForm.setSizaiList(sizaiList);

            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @GetMapping("/delever/init")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> deleverInit(HttpSession session){
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        OrderDeliverForm orderDeliverForm = new OrderDeliverForm();
        List<DepartmentModel> deptList = orderDeliverService.deptList();
        orderDeliverForm.setDeptList(deptList);

        final String DEFAULT_DEPT_ID = "0001";
        List<SizaiModel> sizaiList = orderDeliverService.orderedList(DEFAULT_DEPT_ID);
        orderDeliverForm.setSizaiList(sizaiList);

        if (orderDeliverForm.getSizaiList() == null || orderDeliverForm.getSizaiList().isEmpty()) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("指定部署の購入資材はありません。"));
        }else {

            session.setAttribute("deptId", orderDeliverForm.getDeptId());
            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @PostMapping("/delever/search")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> deleverSearch(@RequestBody OrderDeliverForm orderDeliverForm, HttpSession session){
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        List<DepartmentModel> deptList = orderDeliverService.deptList();
        orderDeliverForm.setDeptList(deptList);

        List<SizaiModel> sizaiList = orderDeliverService.orderedList(orderDeliverForm.getDeptId());
        orderDeliverForm.setSizaiList(sizaiList);

        if (orderDeliverForm.getSizaiList() == null || orderDeliverForm.getSizaiList().isEmpty()) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("指定部署の購入資材はありません。"));
        }else {

            session.setAttribute("deptId", orderDeliverForm.getDeptId());
            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @PostMapping("/delever/act")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> deleverAct(@RequestBody OrderDeliverForm orderDeliverForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        String deptIdStr = orderDeliverForm.getDeptId();
        if (deptIdStr == null || deptIdStr.isBlank()) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("部署IDが指定されていません。"));
        }

        boolean flag = orderDeliverService.deleverAct(orderDeliverForm.getId());

        if (!flag) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("納品失敗しました。"));
        }else {
            orderDeliverForm.setDeptId((String) session.getAttribute("deptId"));
            List<DepartmentModel> deptList = orderDeliverService.deptList();
            List<SizaiModel> sizaiList = orderDeliverService.orderedList(orderDeliverForm.getDeptId());

            orderDeliverForm.setDeptList(deptList);
            orderDeliverForm.setSizaiList(sizaiList);

            return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
        }
    }

    @GetMapping("/delever/qr")
    public ResponseEntity<ApiResponse<OrderDeliverForm>> qrCode(@RequestParam("id") int id,HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"3".equals(user.getRole_class()) || !"5".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        OrderDeliverForm orderDeliverForm = new OrderDeliverForm();
        orderDeliverForm.setId(id);

        return ResponseEntity.ok(ApiResponse.success(orderDeliverForm));
    }
}
